@@ -11,6 +11,7 @@ from playhouse.migrate import SchemaMigrator
 from .base import BaseTestCase
 from .base import IS_MYSQL
 from .base import IS_MYSQL_ADVANCED_FEATURES
+from .base import IS_SQLITE
 from .base import IS_SQLITE_OLD
 from .base import ModelTestCase
 from .base import TestModel
@@ -19,6 +20,7 @@ from .base import requires_models
 from .base import requires_mysql
 from .base import requires_postgresql
 from .base import skip_if
+from .base import skip_unless
 from .base_models import Sample
 from .base_models import Tweet
 from .base_models import User
@@ -1455,6 +1457,9 @@ class TestQueryWithModelInstanceParam(ModelTestCase):
         b1 = FKF_B.create(fk_a_1=a1, fk_a_2=a1)
         b2 = FKF_B.create(fk_a_1=a2, fk_a_2=a2)
 
+        # Ensure that UPDATE works as expected as well.
+        b1.save()
+
         # See also keys.TestFKtoNonPKField test, which replicates much of this.
         args = (b1.fk_a_1, b1.fk_a_1_id, a1, a1.key)
         for arg in args:
@@ -1503,3 +1508,125 @@ class TestModelSelectFromSubquery(ModelTestCase):
                  .order_by(subquery.c.username.desc()))
         self.assertEqual([u.username for u in query], ['u4', 'u0'])
         self.assertTrue(isinstance(query[0], User))
+
+
+class CharPK(TestModel):
+    id = CharField(primary_key=True)
+    name = CharField(unique=True)
+
+    def __str__(self):
+        return self.name
+
+class CharFK(TestModel):
+    id = IntegerField(primary_key=True)
+    cpk = ForeignKeyField(CharPK, field=CharPK.name)
+
+
+class TestModelConversionRegression(ModelTestCase):
+    requires = [CharPK, CharFK]
+
+    def test_model_conversion_regression(self):
+        cpks = [CharPK.create(id=str(i), name='u%s' % i) for i in range(3)]
+
+        query = CharPK.select().where(CharPK.id << cpks)
+        self.assertEqual(sorted([c.id for c in query]), ['0', '1', '2'])
+
+        query = CharPK.select().where(CharPK.id.in_(list(CharPK.select())))
+        self.assertEqual(sorted([c.id for c in query]), ['0', '1', '2'])
+
+    def test_model_conversion_fk_retained(self):
+        cpks = [CharPK.create(id=str(i), name='u%s' % i) for i in range(3)]
+        cfks = [CharFK.create(id=i + 1, cpk='u%s' % i) for i in range(3)]
+
+        c0, c1, c2 = cpks
+        query = CharFK.select().where(CharFK.cpk << [c0, c2])
+        self.assertEqual(sorted([f.id for f in query]), [1, 3])
+
+
+class FKN_A(TestModel): pass
+class FKN_B(TestModel):
+    a = ForeignKeyField(FKN_A, null=True)
+
+class TestSetFKNull(ModelTestCase):
+    requires = [FKN_A, FKN_B]
+
+    def test_set_fk_null(self):
+        a1 = FKN_A.create()
+        a2 = FKN_A()
+        b1 = FKN_B(a=a1)
+        b2 = FKN_B(a=a2)
+
+        self.assertTrue(b1.a is a1)
+        self.assertTrue(b2.a is a2)
+        b1.a = b2.a = None
+        self.assertTrue(b1.a is None)
+        self.assertTrue(b2.a is None)
+
+
+class TestWeirdAliases(ModelTestCase):
+    requires = [User]
+
+    @skip_if(IS_MYSQL)  # mysql can't do anything normally.
+    def test_weird_aliases(self):
+        User.create(username='huey')
+        def assertAlias(s, expected):
+            query = User.select(s).dicts()
+            row = query[0]
+            self.assertEqual(list(row)[0], expected)
+
+        # When we explicitly provide an alias, use that.
+        assertAlias(User.username.alias('"username"'), '"username"')
+        assertAlias(User.username.alias('(username)'), '(username)')
+        assertAlias(User.username.alias('user(name)'), 'user(name)')
+        assertAlias(User.username.alias('(username"'), '(username"')
+        assertAlias(User.username.alias('"username)'), '"username)')
+        assertAlias(fn.LOWER(User.username).alias('user (name)'), 'user (name)')
+
+        # Here peewee cannot tell that an alias was given, so it will attempt
+        # to clean-up the column name returned by the cursor description.
+        assertAlias(SQL('"t1"."username" AS "user name"'), 'user name')
+        assertAlias(SQL('"t1"."username" AS "user (name)"'), 'user (name')
+        assertAlias(SQL('"t1"."username" AS "(username)"'), 'username')
+        assertAlias(SQL('"t1"."username" AS "x.y.(username)"'), 'username')
+        if IS_SQLITE:
+            assertAlias(SQL('LOWER("t1"."username")'), 'username')
+
+
+
+class NDF(TestModel):
+    key = CharField(primary_key=True)
+    date = DateTimeField(null=True)
+
+class TestBulkUpdateAllNull(ModelTestCase):
+    requires = [NDF]
+
+    @skip_unless(IS_SQLITE or IS_MYSQL, 'postgres cannot do this properly')
+    def test_bulk_update_all_null(self):
+        n1 = NDF.create(key='n1', date=datetime.datetime(2021, 1, 1))
+        n2 = NDF.create(key='n2', date=datetime.datetime(2021, 1, 2))
+        rows = [NDF(key=key, date=None) for key in ('n1', 'n2')]
+        NDF.bulk_update(rows, fields=['date'])
+
+        query = NDF.select().order_by(NDF.key).tuples()
+        self.assertEqual([r for r in query], [('n1', None), ('n2', None)])
+
+
+class CQA(TestModel):
+    a = TextField()
+    b = TextField()
+
+
+class TestSelectFromUnion(ModelTestCase):
+    requires = [CQA]
+
+    def test_select_from_union(self):
+        CQA.insert_many([('a%d' % i, 'b%d' % i) for i in range(10)]).execute()
+
+        q1 = CQA.select(CQA.a).order_by(CQA.id).limit(3)
+        q2 = CQA.select(CQA.b).order_by(CQA.id).limit(3)
+
+        wq1 = q1.select_from(SQL('*'))
+        wq2 = q2.select_from(SQL('*'))
+        union = wq1 | wq2
+        data = [val for val, in union.tuples()]
+        self.assertEqual(sorted(data), ['a0', 'a1', 'a2', 'b0', 'b1', 'b2'])
