@@ -5,6 +5,7 @@ from peewee import NodeList
 
 from .base import BaseTestCase
 from .base import get_in_memory_db
+from .base import IS_CRDB
 from .base import IS_SQLITE
 from .base import ModelDatabaseTestCase
 from .base import ModelTestCase
@@ -40,6 +41,14 @@ class TMConstraints(TestModel):
     value = TextField(collation='NOCASE')
 
 
+class TMNamedConstraints(TestModel):
+    fk = ForeignKeyField('self', null=True, constraint_name='tmc_fk')
+    k = TextField()
+    v = IntegerField(constraints=[Check('v in (1, 2)')])
+    class Meta:
+        constraints = [Check('k != \'kx\'', name='chk_k')]
+
+
 class CacheData(TestModel):
     key = TextField(unique=True)
     value = TextField()
@@ -67,7 +76,8 @@ Article.add_index(SQL('CREATE INDEX "article_foo" ON "article" ("flags" & 3)'))
 class TestModelDDL(ModelDatabaseTestCase):
     database = get_in_memory_db()
     requires = [Article, CacheData, Category, Note, Person, Relationship,
-                TMUnique, TMSequence, TMIndexes, TMConstraints, User]
+                TMUnique, TMSequence, TMIndexes, TMConstraints,
+                TMNamedConstraints, User]
 
     def test_database_required(self):
         class MissingDB(Model):
@@ -308,6 +318,46 @@ class TestModelDDL(ModelDatabaseTestCase):
              '"key" TEXT NOT NULL PRIMARY KEY, '
              '"value" TEXT NOT NULL)')])
 
+    def test_strict_tables(self):
+        class Strict(TestModel):
+            key = TextField(primary_key=True)
+            value = TextField()
+
+            class Meta:
+                database = self.database
+                strict_tables = True
+
+        self.assertCreateTable(Strict, [
+            ('CREATE TABLE "strict" ('
+             '"key" TEXT NOT NULL PRIMARY KEY, '
+             '"value" TEXT NOT NULL) STRICT')])
+
+        # Subclasses *do* inherit "strict_tables" setting.
+        class SubStrict(Strict): pass
+
+        self.assertCreateTable(SubStrict, [
+            ('CREATE TABLE "sub_strict" ('
+             '"key" TEXT NOT NULL PRIMARY KEY, '
+             '"value" TEXT NOT NULL) STRICT')])
+
+    def test_without_rowid_strict(self):
+        class KV(TestModel):
+            key = TextField(primary_key=True)
+            class Meta:
+                database = self.database
+                strict_tables = True
+                without_rowid = True
+
+        self.assertCreateTable(KV, [
+            ('CREATE TABLE "kv" ("key" TEXT NOT NULL PRIMARY KEY) '
+             'STRICT, WITHOUT ROWID')])
+
+        class SKV(KV):
+            pass
+
+        self.assertCreateTable(SKV, [
+            ('CREATE TABLE "skv" ("key" TEXT NOT NULL PRIMARY KEY) STRICT')])
+
     def test_db_table(self):
         class A(TestModel):
             class Meta:
@@ -472,10 +522,23 @@ class TestModelDDL(ModelDatabaseTestCase):
 
         self.assertCreateTable(TMConstraints, [
             ('CREATE TABLE "tm_constraints" ('
-             '"id" INTEGER NOT NULL PRIMARY KEY,'
-             ' "data" INTEGER CHECK (data < 5), '
+             '"id" INTEGER NOT NULL PRIMARY KEY, '
+             '"data" INTEGER CHECK (data < 5), '
              '"value" TEXT NOT NULL COLLATE NOCASE)')])
 
+        self.assertCreateTable(TMNamedConstraints, [
+            ('CREATE TABLE "tm_named_constraints" ('
+             '"id" INTEGER NOT NULL PRIMARY KEY, '
+             '"fk_id" INTEGER, '
+             '"k" TEXT NOT NULL, '
+             '"v" INTEGER NOT NULL '
+             'CHECK (v in (1, 2)), '
+             'CONSTRAINT "tmc_fk" FOREIGN KEY ("fk_id") '
+             'REFERENCES "tm_named_constraints" ("id"), '
+             'CONSTRAINT "chk_k" CHECK (k != \'kx\'))'),
+            ('CREATE INDEX "tm_named_constraints_fk_id" '
+             'ON "tm_named_constraints" ("fk_id")')])
+
     def test_index_name_truncation(self):
         class LongIndex(TestModel):
             a123456789012345678901234567890 = CharField()
@@ -758,3 +821,90 @@ class TestTruncateTable(ModelTestCase):
 
         User.truncate_table()
         self.assertEqual(User.select().count(), 0)
+
+
+class TestNamedConstraintsIntegration(ModelTestCase):
+    requires = [TMNamedConstraints]
+
+    def setUp(self):
+        super(TestNamedConstraintsIntegration, self).setUp()
+        if IS_SQLITE:
+            self.database.pragma('foreign_keys', 'on')
+
+    def test_named_constraints_integration(self):
+        t = TMNamedConstraints.create(k='k1', v=1)  # Sanity test.
+        fails = [
+            {'fk': t.id - 1, 'k': 'k2', 'v': 1},  # Invalid fk.
+            {'fk': t.id, 'k': 'k3', 'v': 0},  # Invalid val.
+            {'fk': t.id, 'k': 'kx', 'v': 1}]  # Invalid key.
+        for f in fails:
+            # MySQL may use OperationalError.
+            with self.assertRaises((IntegrityError, OperationalError)):
+                with self.database.atomic() as tx:
+                    TMNamedConstraints.create(**f)
+
+        self.assertEqual(len(TMNamedConstraints), 1)
+
+
+class TMKV(TestModel):
+    key = CharField()
+    value = IntegerField()
+    extra = IntegerField()
+
+class TMKVNew(TestModel):
+    key = CharField()
+    val = IntegerField()
+    class Meta:
+        primary_key = False
+        table_name = 'tmkv_new'
+
+
+class TestCreateTableAsSQL(ModelDatabaseTestCase):
+    database = get_in_memory_db()
+    requires = [TMKV]
+
+    def test_create_table_as_sql(self):
+        query = (TMKV
+                 .select(TMKV.key, TMKV.value.alias('val'))
+                 .where(TMKV.extra < 4))
+        ctx = TMKV._schema._create_table_as('tmkv_new', query)
+        self.assertSQL(ctx, (
+            'CREATE TABLE IF NOT EXISTS "tmkv_new" AS '
+            'SELECT "t1"."key", "t1"."value" AS "val" FROM "tmkv" AS "t1" '
+            'WHERE ("t1"."extra" < ?)'), [4])
+
+        ctx = TMKV._schema._create_table_as(('alt', 'tmkv_new'), query)
+        self.assertSQL(ctx, (
+            'CREATE TABLE IF NOT EXISTS "alt"."tmkv_new" AS '
+            'SELECT "t1"."key", "t1"."value" AS "val" FROM "tmkv" AS "t1" '
+            'WHERE ("t1"."extra" < ?)'), [4])
+
+
+class TestCreateTableAs(ModelTestCase):
+    requires = [TMKV]
+
+    def tearDown(self):
+        try:
+            TMKVNew.drop_table(safe=True)
+        except:
+            pass
+        super(TestCreateTableAs, self).tearDown()
+
+    def test_create_table_as(self):
+        TMKV.insert_many([('k%02d' % i, i, i) for i in range(10)]).execute()
+
+        query = (TMKV
+                 .select(TMKV.key, TMKV.value.alias('val'))
+                 .where(TMKV.extra < 4))
+        query.create_table('tmkv_new', safe=True)
+
+        expected = ['key', 'val']
+        if IS_CRDB: expected.append('rowid')  # CRDB adds this.
+
+        self.assertEqual(
+            [col.name for col in self.database.get_columns('tmkv_new')],
+            expected)
+
+        query = TMKVNew.select().order_by(TMKVNew.key)
+        self.assertEqual([(r.key, r.val) for r in query],
+                         [('k00', 0), ('k01', 1), ('k02', 2), ('k03', 3)])
