@@ -3,6 +3,7 @@ import operator
 from peewee import *
 from playhouse.shortcuts import *
 
+from .base import BaseTestCase
 from .base import DatabaseTestCase
 from .base import ModelTestCase
 from .base import TestModel
@@ -661,3 +662,78 @@ class TestResolveMultiModelQuery(ModelTestCase):
             self.assertEqual(row.__class__, exp_row.__class__)
             self.assertEqual(row.key, exp_row.key)
             self.assertEqual(row.value, exp_row.value)
+
+
+ts_database = get_in_memory_db()
+
+class TSBase(Model):
+    class Meta:
+        database = ts_database
+        model_metadata_class = ThreadSafeDatabaseMetadata
+
+class TSReg(TSBase):
+    key = TextField()
+
+class TestThreadSafeDatabaseMetadata(BaseTestCase):
+    def setUp(self):
+        super(TestThreadSafeDatabaseMetadata, self).setUp()
+        ts_database.create_tables([TSReg])
+
+    def test_threadsafe_database_metadata(self):
+        self.assertTrue(isinstance(TSReg._meta, ThreadSafeDatabaseMetadata))
+        self.assertEqual(TSReg._meta.database, ts_database)
+
+        t1 = TSReg.create(key='k1')
+        t1_db = TSReg.get(TSReg.key == 'k1')
+        self.assertEqual(t1.id, t1_db.id)
+
+    def test_swap_database(self):
+        d1 = get_in_memory_db()
+        d2 = get_in_memory_db()
+
+        class M(TSBase):
+            pass
+
+        def swap_db():
+            self.assertEqual(M._meta.database, ts_database)
+            d1.bind([M])
+            self.assertEqual(M._meta.database, d1)
+            with d2.bind_ctx([M]):
+                self.assertEqual(M._meta.database, d2)
+            self.assertEqual(M._meta.database, d1)
+
+        self.assertEqual(M._meta.database, ts_database)
+
+        # From a separate thread, swap the database and verify it works
+        # correctly.
+        t = threading.Thread(target=swap_db)
+        t.start() ; t.join()
+
+        # In the main thread the original database has not been altered.
+        self.assertEqual(M._meta.database, ts_database)
+
+    def test_preserve_original_db(self):
+        outputs = []
+        d1 = get_in_memory_db()
+        d2 = get_in_memory_db()
+
+        class M(TSBase):
+            class Meta:
+                database = d1
+
+        def swap_db():
+            self.assertTrue(M._meta.database is d1)
+            with d2.bind_ctx([M]):
+                self.assertTrue(M._meta.database is d2)
+            self.assertTrue(M._meta.database is d1)
+            d2.bind([M])  # Now bind to d2 and leave it bound.
+            self.assertTrue(M._meta.database is d2)
+
+        # From a separate thread, swap the database and verify it works
+        # correctly.
+        threads = [threading.Thread(target=swap_db) for _ in range(20)]
+        for t in threads: t.start()
+        for t in threads: t.join()
+
+        # In the main thread the original database has not been altered.
+        self.assertTrue(M._meta.database is d1)
